package descartes.kieker4dql.dqlconnector.filter;

import descartes.kieker4dql.dqlconnector.structures.resources.ResourceMap;
import descartes.kieker4dql.dqlconnector.structures.resources.ResourceMapKey;
import descartes.kieker4dql.dqlconnector.structures.resources.ResourceMapValue;
import kieker.analysis.IProjectContext;
import kieker.analysis.plugin.annotation.InputPort;
import kieker.analysis.plugin.annotation.OutputPort;
import kieker.analysis.plugin.annotation.Plugin;
import kieker.analysis.plugin.filter.AbstractFilterPlugin;
import kieker.common.configuration.Configuration;
import kieker.common.record.IMonitoringRecord;
import kieker.common.record.system.ResourceUtilizationRecord;

@Plugin(
		name = "Resource filter",
		description = "Extracts resource-information from incoming monitoring records",
		outputPorts = {
				@OutputPort(name = ResourceFilter.OUTPUT_PORT_NAME,
						description = "Outputs ResourceUtilizationRecords",
						eventTypes = { IMonitoringRecord.class }),
				@OutputPort(name = ResourceFilter.OUTPUT_PORT_NAME_OTHER,
						description = "Outputs untouched information",
						eventTypes = { IMonitoringRecord.class })}
		)

public class ResourceFilter  extends AbstractFilterPlugin {

	public static final String INPUT_PORT_NAME = "newMonitoringRecord";
	public static final String OUTPUT_PORT_NAME = "forwardingResUtRecord";
	public static final String OUTPUT_PORT_NAME_OTHER = "untouchedIMonitoringRecords";

	public ResourceFilter(final Configuration configuration, final IProjectContext projectContext) {
		super(configuration, projectContext);
	}

	@InputPort(
			name = ResourceFilter.INPUT_PORT_NAME,
			description = "Extract resource-information from monitoring record",
			eventTypes = { IMonitoringRecord.class })

	public void runFilter(final Object record) {
		if (record instanceof ResourceUtilizationRecord) {
			final ResourceUtilizationRecord resUtRecord = (ResourceUtilizationRecord) record;
			
			final long timestamp = resUtRecord.getLoggingTimestamp();
			final String hostName = resUtRecord.getHostname();
			final String resourceName = resUtRecord.getResourceName();
			final double utilization = resUtRecord.getUtilization();
			
			
			ResourceMapKey key = new ResourceMapKey(hostName, resourceName);
			ResourceMapValue value = new ResourceMapValue(timestamp,utilization);
			ResourceMap.getInstance().put(key, value);
			super.deliver(OUTPUT_PORT_NAME, resUtRecord);
		} else {
			super.deliver(OUTPUT_PORT_NAME_OTHER, record);
		}
	}


	@Override
	public Configuration getCurrentConfiguration() {
		return new Configuration();
	}
}
